/////////////////////
// SEGÉDFÜGGVÉNYEK //
/////////////////////

// $ segédfüggvény
function $(selector) {
  return document.querySelector(selector);
}

// segedfüggvény korábbi feladatból
function veletlenEgesz(min, max) {
  const veletlen = Math.random();
  const tartomany = max - min + 1;
  return Math.trunc(veletlen * tartomany) + min;
}

// toronyrajzoló segédfüggvény korábbi feladatból
// Egy torony kirajzolása
function rajzolTorony(x, y, szelesseg, magassag, szin) {
  const ablakMagassag = 6;
  const ablakYtavolsag = 6;
  const ablakSzelesseg = 16;
  const ablakXtavolsag = 4;

  // ház alapja
  rajz.beginPath();
  rajz.fillStyle = szin;
  rajz.fillRect(x + 1, y + 1, szelesseg - 2, magassag - 2);
  rajz.closePath();

  // ablaksorok és az egy sorban lévő ablakok száma
  const sorokSzama = Math.floor((magassag - ablakYtavolsag) / (ablakMagassag + ablakYtavolsag));
  const ablakokSzama = Math.floor((szelesseg - ablakXtavolsag) / (ablakSzelesseg + ablakXtavolsag));

  // ablakok rajzolása
  let ablakSzam = magassag % ablakokSzama;
  // rajzolunk egy sor ablakot ablakokat (10-10 pixel a szélén üres)
  for (let i = 0; i < sorokSzama; i++) {
    // rajzolunk ablakokat (10-10 pixel a szélén üres)
    for (let j = 0; j < ablakokSzama; j++) {
      rajz.beginPath();
      // minden 13. és 17. ablak világít
      if (ablakSzam % 13 == 0 || ablakSzam % 17 == 0) {
        rajz.fillStyle = "#FFFF9A";
      } else {
        rajz.fillStyle = "#131522";
      }
      rajz.fillRect(x + ablakXtavolsag + j * (ablakSzelesseg + ablakXtavolsag), 
                    y + ablakYtavolsag + i * (ablakMagassag + ablakYtavolsag),
                    ablakSzelesseg, ablakMagassag);
      rajz.closePath();
      ablakSzam++;
    }
  }
}

/////////////////////
// BEÁLLÍTÁSOK     //
/////////////////////

// a vászon méretei
const szelesseg = 640;
const magassag = 360;

// Gravitációs tényező
const g = 0.2;
// helikopter x koordinátája
const x = 50;
// a helikopter víszintes sebessége (valójában a tornyok sebessége)
const vx = 3;
// a helikopter függőleges gyorsulása gombnyomásra
const ay = 7;
// hány ezredmásodpercenként frissüljön a kép
const frissitesiIdo = 17;
// egy torony szélessége
const toronySzelesseg = 85;
// távolság a felső és az alsó torony között
const toronyTavolsag = 120;
// milyen időközönként jöjjön új torony
const ujToronyIntervallum = 2000;

// a helikopter képe
const helikopterKep = new Image();
helikopterKep.src = "helikopter.png";
const helikopterMagassag = 39;
const helikopterSzelesseg = 55;

/////////////////////
// VÁSZONBEÁLLÍTÁS //
/////////////////////

let vaszon = $("canvas");
let rajz = vaszon.getContext("2d");
vaszon.width = szelesseg;
vaszon.height = magassag;

/////////////////////
// ÁLLAPOTTÉR      //
/////////////////////

let allapot; // a játék éppen aktuális állapota (jatek, kezd, vesztettel)

let y;        // a helikopter függőleges helye
let vy;       // a helikopter függőleges sebessége

let tornyok; // a tornyok helye;
let pontszam; // pontszám
let idozito; // a játék főidőzítője
let toronyIdozito; // a tornyok generálásának időzítője

// a játék kirajzolása
function kirajzol() {
  rajz.clearRect(0, 0, szelesseg, magassag); // vászon törlése
  rajzolHelikopter(y);
  for (let torony of tornyok) {
    rajzolToronyPar(torony);
  }
  rajzolPontszam();
}

// kezdőállapot beállítása új játék indításánál
function kezdoAllapot() {
  allapot = "kezd";
  y = magassag / 2;
  vy = 0;
  tornyok = [];
  pontszam = 0;
}

// új torony létrehozása
function ujTorony() {
  tornyok.push({
    hely: szelesseg, // létrehozáskor a pálya szélén van
    magassag: veletlenEgesz(magassag / 10, magassag / 10 * 6) // véletlen magasság
  });
}

// a következő játékállapot előállítása
function kovetkezoAllapot() {
  vy += g;
  y += vy;
  for (let cso of tornyok) {
    cso.hely -= vx;
    if (cso.hely < -toronySzelesseg) {
      pontszam++;
      tornyok.splice(tornyok.indexOf(cso), 1);
    }
  }
  vegeEllenoriz();
  kirajzol();
}

// a játék végének ellenőrzése
function vegeEllenoriz() {
  if (y >= magassag) {
    allapot = "vesztettel";
    clearInterval(toronyIdozito);
    clearInterval(idozito);
  }

  for (let cso of tornyok) {
    if (x + helikopterSzelesseg > cso.hely && x < cso.hely + toronySzelesseg) {
      if (y < cso.magassag || y + helikopterMagassag > cso.magassag + toronyTavolsag) {
        allapot = "vesztettel";
        clearInterval(toronyIdozito);
        clearInterval(idozito);
      }
    }
  }
}

/////////////////////
// ESEMÉNYKEZELŐK  //
/////////////////////

// alap rajz elkeszitese
function inicializal() {
  kezdoAllapot();
  kirajzol();
}

// az inicializal függvény hozzárendelése az oldal betöltéséhez
window.addEventListener("load", inicializal);

// kezdőállapot generálása
function start() {
  // kezdő állapot beállítása
  kezdoAllapot();
  // beállítani, hogy fut a játék
  allapot = "jatek";
  // játék elindítása
  idozito = setInterval(kovetkezoAllapot, frissitesiIdo);
  toronyIdozito = setInterval(ujTorony, ujToronyIntervallum);
}

// gombnyomás
function gombLe() { 
  // ha nem fut a játék indísd el
  if (allapot == "kezd" || allapot == "vesztettel") {
    start();
  }
  vy -= ay;
  // függőleges sebesség maximalizálása
  if (vy <= -ay) {
    vy = -ay;
  }
}

// a gombLe függvény hozzárendelése a gombnyomásokhoz
window.addEventListener("keypress", gombLe);

/////////////////////
// RAJZOLÓ         //
/////////////////////

// a helikopter kirajzolása
function rajzolHelikopter() {
  rajz.drawImage(helikopterKep, x, y);
}

// egy toronypár kirajzolása
function rajzolToronyPar(torony) {  
  let szin;
  // a ház magassága alapján véletlenszerűen a két szín valamelyike
  if (torony.magassag % 2 == 0) {
    szin = "#899499";
  } else {
    szin = "#916B6B";
  }
  // a felső ház kirajzolása
  rajzolTorony(torony.hely, 0, 
            toronySzelesseg, torony.magassag, szin);
  // az alsó ház kirajzolása
  rajzolTorony(torony.hely, torony.magassag + toronyTavolsag, 
            toronySzelesseg, magassag - torony.magassag - toronyTavolsag, szin);
}

// ponszám kiírása
function rajzolPontszam() {
  rajz.beginPath();
  rajz.fillStyle = "#fff";
  rajz.font = "32px Consolas";
  rajz.fillText(pontszam, 10, 30);
  rajz.closePath();
}